"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const zrim_base_objects_1 = require("zrim-base-objects");
const zrim_errors_1 = require("zrim-errors");
const _ = require("lodash");
/**
 * Simple implementation of the postgresql manager
 */
class SimplePostGreSqlManager extends zrim_base_objects_1.SimpleConnectableObject {
    /**
     * @inheritDoc
     */
    get flags() {
        return this.getFlags();
    }
    /**
     * @inheritDoc
     */
    get database() {
        return this.properties.database;
    }
    /**
     * @inheritDoc
     */
    getFlags() {
        return _.merge({}, this.properties.flags);
    }
    /**
     * @inheritDoc
     */
    isReadOnly() {
        const flags = this.getFlags();
        const readOnly = _.get(flags, 'readOnly', false) === true;
        return readOnly;
    }
    /**
     * @inheritDoc
     */
    async initialize(options) {
        return await super.initialize(options);
    }
    /**
     * @inheritDoc
     */
    async checkDataBaseHealth(options) {
        const logger = (_.get(options, 'inputContext.logger') || this.logger).of({ prefixes: ['pushMessages'] });
        if (!this.isReady()) {
            const activeStateNames = this.getActivatedStateNames().join(',');
            logger.debug(`Manager not ready for a health check. Active state names are '${activeStateNames}'`);
            throw new zrim_errors_1.common.NotReadyError(`Not ready. Active states '${activeStateNames}'`);
        }
        const checkOptions = {
            checkOperations: options.checkOperations,
            inputContext: options.inputContext,
            healthCheckOptions: options.healthCheckOptions
        };
        let client;
        try {
            client = await this.properties.database.connect();
            logger.debug("Get connection success");
            checkOptions.client = client;
        }
        catch (error) {
            logger.debug({ error }, `Failed to connection : ${error.message}`);
            checkOptions.connectionError = error;
        }
        try {
            const { healthy, checkHealthResult } = await this._handleCheckDataBaseHealth(checkOptions);
            logger.debug("Release the connection");
            if (client) {
                client.done();
            }
            client = undefined;
            return {
                healthy,
                checkHealthResult
            };
        }
        catch (error) {
            logger.debug({ error }, `Fails during the health check: ${error.message}`);
            if (client) {
                logger.debug("Release the connection");
                client.done();
            }
            throw error;
        }
    }
    /**
     * @inheritDoc
     */
    async _handleInitialization(options, ...args) {
        await super._handleInitialization(options, ...args);
        if (!/^postgresql:\/\//.test(options.connectionUrl)) {
            throw new zrim_errors_1.common.IllegalArgumentError(`Invalid connection url '${options.connectionUrl}'`);
        }
        const pgsqlDriver = require('pg-promise')(options.nativeDriverOptions || {});
        // Now copy the options
        this.properties.pgsqlDriver = pgsqlDriver;
        this.properties.pgsqlOptions = {
            connectionUrl: options.connectionUrl
        };
        this.properties.flags = _.merge({}, options.flags);
    }
    /**
     * @inheritDoc
     */
    async _handleDisconnection(...args) {
        const logger = this.logger;
        logger.debug({ prefixes: ['_handleDisconnection'] }, "Close the database connection pool");
        this.properties.database.$pool.end();
        delete this.properties.database;
    }
    /**
     * Check if the connection is valid
     * @param options
     */
    async _checkDataBaseConnection(options) {
        const logger = this.logger.of({ prefixes: ['_checkDataBaseConnection'] });
        try {
            logger.debug("Do a simple SELECT 1");
            await options.client.query('SELECT 1');
            logger.debug("Execution succeed");
            return {
                valid: true
            };
        }
        catch (error) {
            logger.warn({ error }, `Failed to query the database: ${error.message}`);
            return {
                valid: false
            };
        }
    }
    /**
     * Check if teh database is initialized
     * @param options
     */
    async _isDataBaseInitialize(options) {
        const logger = this.logger.of({ prefixes: ['_isDataBaseInitialize'] });
        logger.debug("Not configured.");
        return {
            initialized: true
        };
    }
    async _handleDataBaseInitialization(options) {
        const logger = this.logger.of({ prefixes: ['_handleDataBaseInitialization'] });
        logger.debug("Nothing to initialize");
        return {};
    }
    /**
     * @inheritDoc
     */
    async _handleConnection(...args) {
        const logger = this.logger.of({ prefixes: ['_handleConnection'] });
        logger.debug(`Create the database`);
        const database = this.properties.pgsqlDriver(this.properties.pgsqlOptions.connectionUrl);
        let client;
        try {
            logger.debug(`Call the connect`);
            client = await database.connect();
        }
        catch (error) {
            logger.debug({ error }, `Failed to connection : ${error.message}`);
            throw error;
        }
        try {
            logger.debug(`Call the check handler`);
            const checkDbResponse = await this._checkDataBaseConnection({
                client
            });
            if (checkDbResponse.valid !== true) {
                logger.debug("Connection invalid");
                throw new zrim_errors_1.common.RequirementsNotSatisfiedError('The connection established is invalid');
            }
            logger.debug("Connection valid");
            const { initialized } = await this._isDataBaseInitialize({
                client
            });
            if (initialized !== true) {
                logger.debug("The database need to be initialized");
                await this._handleDataBaseInitialization({
                    client
                });
                logger.debug(`Database initialization done with success`);
            }
            logger.debug("Connection OK and DataBase initialized");
            logger.debug("Release the connection");
            client.done();
            this.properties.database = database;
        }
        catch (error) {
            if (!(error instanceof zrim_errors_1.common.RequirementsNotSatisfiedError)) {
                logger.debug({ error }, `Check failed: ${error.message}`);
            }
            logger.debug("Release the connection");
            client.done();
            logger.debug("End the database");
            database.$pool.end();
            throw error;
        }
    }
    /**
     * Handle the database health check
     * @param options
     */
    async _handleCheckDataBaseHealth(options) {
        const logger = this.logger.of({ prefixes: ['_handleCheckDataBaseHealth'] });
        if (!options.client) {
            logger.debug(`No client connection provided`);
            return {
                healthy: false,
                error: options.connectionError
            };
        }
        try {
            logger.debug("Do a simple SELECT 1");
            await options.client.query('SELECT 1');
            logger.debug("Execution succeed");
            return {
                healthy: true
            };
        }
        catch (error) {
            logger.debug({ error }, `Failed to query the database: ${error.message}`);
            return {
                healthy: false,
                error
            };
        }
    }
}
exports.SimplePostGreSqlManager = SimplePostGreSqlManager;
//# sourceMappingURL=simple-pgsql-manager.js.map